package com.example.example3;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.UnsupportedEncodingException;

import org.apache.http.HttpResponse;
import org.apache.http.client.ClientProtocolException;
import org.apache.http.client.HttpClient;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.impl.client.DefaultHttpClient;
import org.json.JSONException;
import org.json.JSONObject;

import android.app.Activity;
import android.app.ProgressDialog;
import android.content.Context;
import android.content.Intent;
import android.content.SharedPreferences;
import android.content.SharedPreferences.Editor;
import android.os.AsyncTask;
import android.os.Bundle;
import android.view.Menu;
import android.view.View;
import android.widget.Button;
import android.widget.TextView;

public class MainActivity2 extends Activity {

	private static final String STATUS_KEY = "status";
	private static final String COUNTRY_KEY = "country";
	private static final String IP_KEY = "query";
	private static final String PREFERENCE_NAME = "LastData";
	private static final String CACHE_DATA_KEY = "CacheData";
	private TextView statusCode, country, ipAddress;
	private AsyncTask<String, Integer, JSONObject> ipAddressAsyncTask;
	private ProgressDialog progressDialog;
	private Button gotoNextActivity;
	private SharedPreferences cacheData;
	
	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.activity_main);
		
		statusCode = (TextView)findViewById(R.id.statusCode);
		country = (TextView)findViewById(R.id.country);
		ipAddress = (TextView)findViewById(R.id.ipAddress);
		progressDialog = new ProgressDialog(this);      
		progressDialog.setIndeterminate(false);
		progressDialog.setProgressStyle(ProgressDialog.STYLE_HORIZONTAL);
		progressDialog.setMax(5);

		ipAddressAsyncTask = new AsyncTask<String, Integer, JSONObject>() {            
			@Override
			protected void onPreExecute() {
				super.onPreExecute();
				progressDialog.show();
			}
			@Override
			protected JSONObject doInBackground(String... params) {
				JSONObject result;
				result =queryData(params[0]);
				publishProgress(0);
				for(int i=1;i<=5;i++){
					try{
						Thread.sleep(1000);
					}catch (InterruptedException e) { }
					publishProgress(i);
				}
				return result; 
			}
        	@Override
        	protected void onProgressUpdate(Integer... values) {
        		super.onProgressUpdate(values);
        		progressDialog.setProgress(values[0].intValue());
        	}            
        	@Override
        	protected void onPostExecute(JSONObject result) {
        		super.onPostExecute(result);
        		progressDialog.dismiss();
        		parseAndFillData(result);
        	}
		};
		ipAddressAsyncTask.execute("http://ip-api.com/json");
		
		gotoNextActivity = (Button)findViewById(R.id.gotoNextActivity );
        gotoNextActivity .setOnClickListener(new Button.OnClickListener(){
           @Override
            public void onClick(View v) {
               Intent intent = new Intent();
               intent.setClass(MainActivity2.this, MainActivity.class);
               Bundle bundle = new Bundle();
               bundle.putString(MainActivity.CHANGE_BUTTON_TEXT_KEY,"No Function");
               intent.putExtras(bundle);
               startActivity(intent);
            } 
        });

		cacheData = getSharedPreferences(PREFERENCE_NAME, Context.MODE_PRIVATE);
		// if data exists, we can use the cache data
		if (cacheData.contains(CACHE_DATA_KEY)) {
			String jSONString = cacheData.getString(CACHE_DATA_KEY, null);
			JSONObject jObj = null;
			try {
				jObj = new JSONObject(jSONString);
			} catch (JSONException e) {
				e.printStackTrace();
			}

			if (jObj != null) {
				parseAndFillData("Read From " + CACHE_DATA_KEY, jObj);
			}
		}

	}
	
	private JSONObject queryData(String URL){
		// Network access
        HttpClient lClient = new DefaultHttpClient();
        HttpGet lGetMethod = new HttpGet(URL);
        HttpResponse lResp = null;
        try {
			lResp = lClient.execute(lGetMethod);
		} catch (ClientProtocolException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
        
        if(lResp==null) return null;
        
        // Read JSON data from http response
        ByteArrayOutputStream lBOS = new ByteArrayOutputStream();
        String lInfoStr = null;
        JSONObject jSONObjResponse = null;
        try {
			lResp.getEntity().writeTo(lBOS);
			lInfoStr = lBOS.toString("UTF-8");
			jSONObjResponse = new JSONObject(lInfoStr);
		}catch (UnsupportedEncodingException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
        catch (JSONException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
        return jSONObjResponse;
    }

	private void parseAndFillData(String loadingStatus, JSONObject jObj){
		String sts=null, ctry=null, ip=null;
		
		if(jObj == null) return;
        // parse JSONObject
		try {
			if(loadingStatus!=null)
				sts = loadingStatus;
			else
				sts = jObj.getString(STATUS_KEY);
			ctry = jObj.getString(COUNTRY_KEY);
			ip = jObj.getString(IP_KEY);
		} catch (JSONException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		//show parse data on TextView
		if(statusCode!=null)
			statusCode.setText(sts);
		if(country!=null)
			country.setText(ctry);
		if(ipAddress!=null)
			ipAddress.setText(ip);
	}
	
	private void parseAndFillData(JSONObject jObj){
		if(cacheData!=null && jObj!=null){
            Editor editor = cacheData.edit();
            if(null != editor){
                editor.putString(CACHE_DATA_KEY, jObj.toString());
                editor.apply();
            }
        }
        parseAndFillData(null, jObj);
	}

	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
		// Inflate the menu; this adds items to the action bar if it is present.
		getMenuInflater().inflate(R.menu.main, menu);
		return true;
	}

}
